// PGMImage.cpp: implementation of the CPGMImage class.
//
//////////////////////////////////////////////////////////////////////

#include "PGMImage.h"
#include <string>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPGMImage::CPGMImage()
{
	_data = NULL;
	_width = 0;
	_height = 0;
}

CPGMImage::~CPGMImage()
{
	_deallocate();
		
}

CPGMImage::CPGMImage(unsigned int width, unsigned int height)
{
	_width = width;
	_height = height;

	_allocate();

	_clean();

	
}

bool CPGMImage::loadPGM(char *fileName)
{
	FILE* inFile = NULL;
	char ch;
	bool raw = false;
	int maxValue;
	int i = 0;

	inFile = fopen(fileName,"rt");

	if (inFile == NULL)
	{
		return false;	// Opening error
	}
	
	if (fscanf(inFile, "P%c\n", &ch) != 1 || (ch!='2'&& ch!='5')) 
	{
		fclose(inFile);
		return false;	// Non Valid format error
	}

	if (ch == '5')
		raw = true;

	i += 3;

	// skip comments
	ch = fgetc(inFile);
	while (ch == '#')
	{
		i++;
		do 
		{
			ch = fgetc(inFile);
			i++;
		} 
		while (ch != '\n');   
		ch = fgetc(inFile);    
	}

	ungetc(ch,inFile);
	int width = 0;
	int height = 0;
	
	fscanf(inFile, "%u %u\n", &width, &height);
	fscanf(inFile, "%u\n", &maxValue);
	char _s[255];
	sprintf(_s,"%u %u\n%u\n");
	i += strlen(_s);
		
	if (maxValue > 255)
	{
		fclose(inFile);
		return false;	// Color depth error
	}

	if (resize(width, height) == false)
		return false;

	unsigned char val = 0;

	if (raw)
	{
		fclose(inFile);
		inFile = fopen(fileName,"rb");
		fseek(inFile, i, SEEK_SET);
		for(int y=0; y<_height; y++)
			for (int x=0; x<_width; x++)
			{
				fread((void *) &val, 1, 1, inFile);
				_data[x][y] = (unsigned char)val;
			}
	}
	else
	{
		for(int y=0; y<_height; y++)
			for (int x=0; x<_width; x++)
			{
				fscanf(inFile,"%u",&val);
				_data[x][y] = (unsigned char)val;
			}
	
	}
	
	fclose(inFile);
	return true;
}

bool CPGMImage::savePGM(char *fileName, bool raw)
{
	FILE *outFile;
	if (!raw)
	{
		outFile = fopen(fileName,"wt");

		if (outFile == NULL)
		{
			return false;	// Opening error
		}

		fprintf(outFile,"P2\n");

		
		fprintf(outFile,"# no comment :-)\n");
		
		fprintf(outFile,"%u %u\n", _width, _height);
		fprintf(outFile,"255\n");
		
		unsigned char val = 0;
		int i = 0;
		char _string[255];

		for(int y=0; y<_height; y++)
			for (int x=0; x<_width; x++)
			{
				val = _data[x][y];
						
				sprintf(_string,"%u ",val);
				i += strlen(_string);
				if (i > 70)
				{
					fputc('\n',outFile);
					i = 0;
				}
				fprintf(outFile,"%u ",val);
			}

		fclose(outFile);
	}
	else
	{
		outFile = fopen(fileName,"wb");
		if (outFile == NULL)
		{
			return false;	// Opening error
		}
		_writeBin(outFile,"P5\n");

		
		_writeBin(outFile,"# no comment :-)\n");
		char _s[255];
		sprintf(_s,"%u %u\n", _width, _height);
		_writeBin(outFile, _s);
		_writeBin(outFile,"255\n");

		unsigned char val = 0;
		
		for(int y=0; y<_height; y++)
			for (int x=0; x<_width; x++)
			{
				val = _data[x][y];
				fwrite((void *) &val, 1, 1, outFile);
			}
		fclose(outFile);
	}
	return true;
}

bool CPGMImage::resize(unsigned int width, unsigned int height)
{
	_deallocate();

	_width = width;
	_height = height;

	if (_allocate() == false)
		return false;

	_clean();
	return true;
}

bool CPGMImage::setPixel(unsigned int x, unsigned int y, unsigned char value)
{
	if ((x >= _width) || (y >= _height))
		return false;	// out of size limits error

	_data[x][y] = value;

	return true;
}

bool CPGMImage::getPixel(unsigned int x, unsigned int y, unsigned char* value)
{
	if ((x >= _width) || (y >= _height))
		return false;	// out of size limits error

	*value = _data[x][y];
	
	return true;
}



bool CPGMImage::_allocate()
{
	if (_data != NULL)
		return false;

	_data = new unsigned char* [_width];	
	if (_data == NULL)
		return false;

	for (unsigned int x=0; x<_width;x++)
	{
		_data[x] = NULL;
		_data[x] = new unsigned char[_height]; 
		if (_data[x] == NULL)
			return false;
	}

	return true;
}

void CPGMImage::_deallocate()
{
	if (_data != NULL)
	{
		for (unsigned int x=0; x<_width;x++)
			delete [] _data[x];	
		delete [] _data;		
	}

	_data = NULL;
}

void CPGMImage::_clean()
{
	if (_data == NULL)
		return;

	for (unsigned int y=0; y<_height; y++)
		for (unsigned int x=0; x<_width; x++)
			_data[x][y] = 0;
}

void CPGMImage::_writeBin(FILE *f, char *str)
{
	int l = strlen(str);
	char ch;
	for (int i=0; i<l; i++)
	{
		ch = str[i];
		fwrite((void *)&ch,1,1,f);
	}

}

unsigned int CPGMImage::getWidth()
{
	if (_data == NULL)
		return -1;
	else
		return _width;
}

unsigned int CPGMImage::getHeight()
{
	if (_data == NULL)
		return -1;
	else
		return _height;
}
